@@ -1,80 +1,103 @@
 package Dancer2::Plugin::Cache::CHI;
 our $AUTHORITY = 'cpan:YANICK';
 # ABSTRACT: Dancer plugin to cache response content (and anything else)
-$Dancer2::Plugin::Cache::CHI::VERSION = '1.7.1';
+$Dancer2::Plugin::Cache::CHI::VERSION = '2.0.0';
 use strict;
 use warnings;
 use Carp;
 use CHI;
+use List::MoreUtils qw/ any /;
 
 use Dancer2::Plugin;
 
-register_hook 'before_create_cache';
+plugin_hooks 'before_create_cache';
 
+# actually hold the ref to the args
+has _cache_page => (
+    is        => 'rw',
+    clearer   => 'clear_cache_page',
+    predicate => 'has_cache_page',
+);
 
-my %cache;
-my $cache_page; # actually hold the ref to the args
-my $cache_page_key_generator = sub {
-    return $_[0]->app->request->path;
-};
+has cache_page_key_generator => (
+    is             => 'rw',
+    lazy           => 1,
+    plugin_keyword => 1,
+    default        => sub {
+        sub { $_[0]->app->request->path }
+    },
+);
 
-on_plugin_import {
-    my $dsl = shift;
+sub BUILD {
+    my $plugin = shift;
 
-    $dsl->app->add_hook(
+    $plugin->app->add_hook(
         Dancer2::Core::Hook->new(
             name => 'after',
             code => sub {
-                return unless $cache_page;
+                return unless $plugin->has_cache_page;
 
                 my $resp = shift;
-                cache($dsl)->set( $cache_page_key_generator->($dsl),
+                $plugin->cache->set( $plugin->cache_page_key_generator->($plugin),
                     {
                         status      => $resp->status,
                         headers     => $resp->headers_to_array,
                         content     => $resp->content
                     },
-                    @$cache_page,
+                    @{$plugin->_cache_page},
                 );
 
-                $cache_page = undef;
+                $plugin->clear_cache_page;
+            }
+    ));
+
+    $plugin->app->add_hook(
+        Dancer2::Core::Hook->new(
+            name => 'before',
+            code => sub {
+                $plugin->clear_cache_page;
             }
     ));
 };
 
-register cache => sub {
-    my $dsl = shift;
-    return  $cache{$_[0]//''} ||= _create_cache( $dsl, @_ );
+has _caches => (
+    is      => 'ro',
+    lazy    => 1,
+    default => sub {
+        {}
+    },
+);
+
+has caches_with_honor => (
+    is      => 'ro',
+    default => sub{ {} },
+);
+
+plugin_keywords 'cache';
+
+sub cache {
+    my( $self, $namespace ) = @_;
+    $namespace //= '';
+    return $self->_caches->{$namespace} ||= _create_cache( $self, $namespace, @_ );
 };
 
-my $honor_no_cache = 0;
-
 sub _create_cache {
-    my $dsl = shift;
-    my $namespace = shift;
-    my $args = shift || {};
+    my( $dsl, $namespace, $args ) = @_;
+    $args ||= {};
 
     $dsl->execute_hook( 'plugin.cache_chi.before_create_cache' );
 
-    my %setting = %{
-        $dsl->app->config->{plugins}{'Cache::CHI'} || {}
-    };
+    my %setting = ( %{ $dsl->config }, %$args );
 
-    $setting{namespace} = $namespace if defined $namespace;
+    $setting{namespace} //= $namespace;
 
-    while( my ( $k, $v ) = each %$args ) {
-        $setting{$k} = $v;
-    }
-
-    $honor_no_cache = delete $setting{honor_no_cache}
-        if exists $setting{honor_no_cache};
+    $dsl->caches_with_honor->{$namespace} = delete $setting{honor_no_cache};
 
     return CHI->new(%setting);
 }
 
 
-
-register check_page_cache => sub {
+plugin_keywords check_page_cache => sub {
     my $dsl = shift;
 
     my $hook = sub {
@@ -84,19 +107,17 @@ register check_page_cache => sub {
         # response to Dancer2::Core::Response object for a more correct returning of
         # some HTTP headers (X-Powered-By, Server)
 
-        my $cached = cache($dsl)->get( $cache_page_key_generator->($dsl) )
+        my $cached = cache($dsl)->get( $dsl->cache_page_key_generator->($dsl) )
             or return;
 
-        if ( $honor_no_cache ) {
+        if ( $dsl->caches_with_honor->{''} ) {
 
             my $req =  $dsl->app->request;
 
             no warnings 'uninitialized';
 
-            return if grep {
-                # eval is there to protect from a regression in Dancer 1.31
-                # where headers can be undef
-                eval { $req->header($_) eq 'no-cache' }
+            return if any {
+                $req->header($_) eq 'no-cache'
             } qw/ Cache-Control Pragma /;
         }
 
@@ -105,11 +126,7 @@ register check_page_cache => sub {
                 is_halted => 1,
                 ref $cached eq 'HASH'
                 ?
-                (
-                    status       => $cached->{status},
-                    headers      => $cached->{headers},
-                    content      => $cached->{content}
-                )
+                ( map { $_ => $cached->{$_} } qw/ status headers content / )
                 :
                 ( content => $cached )
             )
@@ -122,41 +139,23 @@ register check_page_cache => sub {
     ));
 };
 
+plugin_keywords cache_page => sub {
+    my ( $plugin, $content, @args ) = @_;
 
-register cache_page => sub {
-    shift;
-
-    my ( $content, @args ) = @_;
-
-    $cache_page = \@args;
+    $plugin->_cache_page(\@args);
 
     return $content;
 };
 
-
-
-register cache_page_key => sub {
-    shift;
-    return $cache_page_key_generator->();
-};
-
-
-register cache_page_key_generator => sub {
-    shift;
-    $cache_page_key_generator = shift;
-};
-
+plugin_keywords cache_page_key => sub { $_[0]->cache_page_key_generator->($_[0]) };
 
 for my $method ( qw/ set get remove clear compute / ) {
-    register 'cache_'.$method => sub {
-        my $dsl = shift;
-        return cache($dsl)->$method( @_ );
+    plugin_keywords "cache_$method" => sub {
+        my $plugin = shift;
+        $plugin->cache->$method(@_);
     }
 }
 
-
-register_plugin for_versions => [1,2];
-
 1;
 
 __END__
@@ -171,7 +170,7 @@ Dancer2::Plugin::Cache::CHI - Dancer plugin to cache response content (and anyth
 
 =head1 VERSION
 
-version 1.7.1
+version 2.0.0
 
 =head1 SYNOPSIS
 
